from thrift.server import TNonblockingServer
from thrift.protocol import TBinaryProtocol
from thrift.transport import TSocket, TTransport
from .FunicornService import Processor
import threading
from ..utils import get_logger, colored_network_name


class ThriftAPI(threading.Thread):
    def __init__(self, funicorn_app, host, port, name='RPC', stat=None, threads=40,
                 timeout=1000, debug=False):
        threading.Thread.__init__(self, daemon=True)
        self.name = name
        self.funicorn_app = funicorn_app
        self.host = host
        self.port = port
        self.stat = stat
        self.threads = threads
        self.debug = debug
        self.logger = get_logger(colored_network_name('RPC'), mode='debug' if debug else 'info')
        self.funicorn_app.register_connection(self)

    def init_connection(self, processor):
        '''
        Initialize all connections with TBinaryProtocol and TFramedTransport

        Parameters
        ----------
        processor : TProcessor
            Processor class has been generated by thrift

        Returns
        -------
        server: TServer
            TNonblocking Server
        '''
        socket = TSocket.TServerSocket(host=self.host, port=self.port)
        prot_fac = TBinaryProtocol.TBinaryProtocolFactory()
        server = TNonblockingServer.TNonblockingServer(processor=processor,
                                                       lsocket=socket,
                                                       inputProtocolFactory=prot_fac,
                                                       threads=self.threads)
        return server

    def init_processor(self, handler):
        processor = Processor(handler)
        return processor

    def run(self):
        processor = self.init_processor(self.funicorn_app)
        server = self.init_connection(processor)
        self.logger.info(
            f'Server is running at http://{self.host}:{self.port}')
        server.serve()
